# delimit ;
clear all;


**********************************************;
*Step 1: generate u0, u1, uv random variables;
**********************************************;

*some parameters;
local sigma_u0 = 1;
local sigma_u1 = 1;
local sigma_uv = 1;
local corr_u01 = 0.5;
local corr_u0v = 0.7;
local corr_u1v = 0.7;

set seed 317;
matrix C = (1, `corr_u01', `corr_u0v' \ `corr_u01', 1, `corr_u1v' \ `corr_u0v', `corr_u1v', 1);
matrix sd = (`sigma_u0',`sigma_u1',`sigma_uv');
drawnorm U0 U1 UV, n(4567) corr(C) sds(sd);
sum U0 U1 UV;
correlate U0 U1 UV;

**********************************************;
*Step 2: generate Z;
**********************************************;

set seed 209;
gen temp = runiform();
gen Z = 1 if temp >= 0.5;
replace Z = 0 if temp < 0.5;
drop temp;

**********************************************;
*Step 3: generate X;
**********************************************;

set seed 328;
gen temp = runiform();
gen X = 1 if temp <= 0.5;
replace X = 2 if temp > 0.5 & temp <= 0.8;
replace X = 3 if temp > 0.8;


**********************************************;
*Step 4: generate remaining variables;
**********************************************;

gen V = -1 + 0.5*Z + UV;

*heterogenous case;
gen Y0 = 1 + U0;
gen Y1 = 5 + U1;

gen Delta = Y1 - Y0;
gen D = 1 if V >= 0;
replace D = 0 if V < 0;
gen Y = D*Y1 + (1-D)*Y0; *observed Y;


**********************************************;
*save dataset;
**********************************************;

*actual ATE;
sum Delta;
local ATE = r(mean);

keep Y D Z X;
compress;
save "C:\Users\mwiswall\Dropbox\teaching\masters metrics\problem_sets\practicedata.dta", replace;


**********************************************;
*Results
**********************************************;

*a) descrip stats;
sum Y D X Z;

*b) OLS;
regress Y D;
gen ols_est = _b[D];

*c) IV;
ivregress 2sls Y (D = Z);
gen iv_est = _b[D];

*c) matching based on X and Z (3 x 2 = 6 total cells);
forvalues x_val = 1(1)3 {;
forvalues z_val = 0(1)1 {;
regress Y D if X == `x_val' & Z == `z_val';
gen match_ATE`x_val'`z_val' = _b[D];
sum Y if X == `x_val' & Z == `z_val';
gen frac`x_val'`z_val' = r(N)/_N;
};
};
gen match_est = 
frac10*match_ATE10 + frac20*match_ATE20 + frac30*match_ATE30
+ frac11*match_ATE11 + frac21*match_ATE21 + frac31*match_ATE31;


*d) control function;

**Step 1: selection model;
probit D Z;
gen delta0_hat = _b[_cons];
gen delta1_hat = _b[Z];
gen Z_delta = delta0_hat + delta1_hat*Z;

**Step 2: control functions;
gen K1 = normalden(-Z_delta) / (1- normal(-Z_delta) );
regress Y K1 if D == 1;
gen cf1 = _b[_cons]  if D == 1;

gen K0 = -normalden(-Z_delta) / normal(-Z_delta);
regress Y K0 if D == 0;
gen cf0 = _b[_cons]  if D == 0;

sum cf1;
gen mean_cf1 = r(mean);
sum cf0;
gen mean_cf0 = r(mean);
gen cf_est = mean_cf1 - mean_cf0;

*compare;
gen actual_ATE = `ATE';
sum actual_ATE ols_est iv_est match_est cf_est;


